//-------------------------------------------------------------------------------------------------
// Wywietlacz alfanumeryczny ze sterownikiem HD44780
// Sterowanie w trybie 4-bitowym bez odczytu flagi zajtoci
// z dowolnym przypisaniem sygnaw sterujcych
// Plik : HD44780.c	
// Mikrokontroler : Atmel AVR
// Kompilator : avr-gcc
// Autor : Radosaw Kwiecie
// rdo : http://radzio.dxp.pl/hd44780/
// Data : 24.03.2007
//-------------------------------------------------------------------------------------------------

#include "lcd.h"
//-------------------------------------------------------------------------------------------------
//
// Funkcja wystawiajca pbajt na magistral danych
//
//-------------------------------------------------------------------------------------------------

void _delay(volatile unsigned int t)
{
volatile unsigned int i;
for(i = 0; i < t; i++) ;
}


void _LCD_OutNibble(unsigned char nibbleToWrite)
{
AT91PS_PIO p_pPio = AT91C_BASE_PIOA;
if(nibbleToWrite & 0x01)
	p_pPio->PIO_SODR|=LCD_DB4;
else
	p_pPio->PIO_CODR|=LCD_DB4;

if(nibbleToWrite & 0x02)
	p_pPio->PIO_SODR |= LCD_DB5;
else
	p_pPio->PIO_CODR |= LCD_DB5;

if(nibbleToWrite & 0x04)
	p_pPio->PIO_SODR |= LCD_DB6;
else
	p_pPio->PIO_CODR |= LCD_DB6;

if(nibbleToWrite & 0x08)
	p_pPio->PIO_SODR |= LCD_DB7;
else
	p_pPio->PIO_CODR |= LCD_DB7;
}
//-------------------------------------------------------------------------------------------------
//
// Funkcja zapisu bajtu do wywietacza (bez rozrnienia instrukcja/dane).
//
//-------------------------------------------------------------------------------------------------
void _LCD_Write(unsigned char dataToWrite)
{
AT91PS_PIO p_pPio = AT91C_BASE_PIOA;
//LCD_E_PORT |= LCD_E;
p_pPio->PIO_SODR|=LCD_E;
_LCD_OutNibble(dataToWrite >> 4);
//LCD_E_PORT &= ~LCD_E;
p_pPio->PIO_CODR|=LCD_E;
//LCD_E_PORT |= LCD_E;
p_pPio->PIO_SODR|=LCD_E;

_LCD_OutNibble(dataToWrite);
//LCD_E_PORT &= ~LCD_E;
p_pPio->PIO_CODR|=LCD_E;
_delay(4000);
}
//-------------------------------------------------------------------------------------------------
//
// Funkcja zapisu rozkazu do wywietlacza
//
//-------------------------------------------------------------------------------------------------
void LCD_WriteCommand(unsigned char commandToWrite)
{
AT91PS_PIO p_pPio = AT91C_BASE_PIOA;
//LCD_RS_PORT &= ~LCD_RS;
p_pPio->PIO_CODR|=LCD_RS;
_LCD_Write(commandToWrite);
}
//-------------------------------------------------------------------------------------------------
//
// Funkcja zapisu danych do pamici wywietlacza
//
//-------------------------------------------------------------------------------------------------
void lcddata(unsigned char dataToWrite)
{
AT91PS_PIO p_pPio = AT91C_BASE_PIOA;
//LCD_RS_PORT |= LCD_RS;
p_pPio->PIO_SODR|=LCD_RS;
_LCD_Write(dataToWrite);
}
//-------------------------------------------------------------------------------------------------
//
// Funkcja wywietlenia napisu na wyswietlaczu.
//
//-------------------------------------------------------------------------------------------------
void lcdtxt(char * text)
{
while(*text)
  lcddata(*text++);
}
//-------------------------------------------------------------------------------------------------
//
// Funkcja ustawienia wsprzdnych ekranowych
//
//-------------------------------------------------------------------------------------------------
void lcdxy(unsigned char x, unsigned char y)
{
LCD_WriteCommand(HD44780_DDRAM_SET | (x + (0x40 * y)));
}
//-------------------------------------------------------------------------------------------------
//
// Funkcja czyszczenia ekranu wywietlacza.
//
//-------------------------------------------------------------------------------------------------
void lcdclr(void)
{
LCD_WriteCommand(HD44780_CLEAR);
_delay(1500);
}
//-------------------------------------------------------------------------------------------------
//
// Funkcja przywrcenia pocztkowych wsprzdnych wywietlacza.
//
//-------------------------------------------------------------------------------------------------
void LCD_Home(void)
{
LCD_WriteCommand(HD44780_HOME);
_delay(1500);
}
//-------------------------------------------------------------------------------------------------
//
// Procedura inicjalizacji kontrolera HD44780.
//
//-------------------------------------------------------------------------------------------------
void lcdinit(void)
{
unsigned char i;

AT91PS_PIO p_pPio = AT91C_BASE_PIOA;
p_pPio->PIO_PER |= LCD_DB4;    
p_pPio->PIO_OER |= LCD_DB4;
p_pPio->PIO_PER |= LCD_DB5;    
p_pPio->PIO_OER |= LCD_DB5;
p_pPio->PIO_PER |= LCD_DB6;    
p_pPio->PIO_OER |= LCD_DB6;
p_pPio->PIO_PER |= LCD_DB7;    
p_pPio->PIO_OER |= LCD_DB7;
p_pPio->PIO_PER |= LCD_E;    
p_pPio->PIO_OER |= LCD_E;
p_pPio->PIO_PER |= LCD_RS;    
p_pPio->PIO_OER |= LCD_RS;

/*
LCD_DB4_DIR |= LCD_DB4; // Konfiguracja kierunku pracy wyprowadze
LCD_DB5_DIR |= LCD_DB5; //
LCD_DB6_DIR |= LCD_DB6; //
LCD_DB7_DIR |= LCD_DB7; //
LCD_E_DIR 	|= LCD_E;   //
LCD_RS_DIR 	|= LCD_RS;  //*/
_delay(65000); // oczekiwanie na ustalibizowanie si napiecia zasilajacego
_delay(65000); // oczekiwanie na ustalibizowanie si napiecia zasilajacego

p_pPio->PIO_CODR|=LCD_RS;
p_pPio->PIO_CODR|=LCD_E;

//LCD_RS_PORT &= ~LCD_RS; // wyzerowanie linii RS
//LCD_E_PORT &= ~LCD_E;  // wyzerowanie linii E

for(i = 0; i < 3; i++) // trzykrotne powtrzenie bloku instrukcji
  {
  //LCD_E_PORT |= LCD_E; //  E = 1
  p_pPio->PIO_SODR|=LCD_E;
  _LCD_OutNibble(0x03); // tryb 8-bitowy
  p_pPio->PIO_CODR|=LCD_E;
  //LCD_E_PORT &= ~LCD_E; // E = 0
  _delay(1800); // czekaj 5ms
  }

//LCD_E_PORT |= LCD_E; // E = 1
p_pPio->PIO_SODR|=LCD_E;
_LCD_OutNibble(0x02); // tryb 4-bitowy
//LCD_E_PORT &= ~LCD_E; // E = 0
p_pPio->PIO_CODR|=LCD_E;

_delay(1800); // czekaj 1ms 
LCD_WriteCommand(HD44780_FUNCTION_SET | HD44780_FONT5x7 | HD44780_TWO_LINE | HD44780_4_BIT); // interfejs 4-bity, 2-linie, znak 5x7
LCD_WriteCommand(HD44780_DISPLAY_ONOFF | HD44780_DISPLAY_OFF); // wyczenie wyswietlacza
LCD_WriteCommand(HD44780_CLEAR); // czyszczenie zawartosi pamieci DDRAM
_delay(1800);
LCD_WriteCommand(HD44780_ENTRY_MODE | HD44780_EM_SHIFT_CURSOR | HD44780_EM_INCREMENT);// inkrementaja adresu i przesuwanie kursora
LCD_WriteCommand(HD44780_DISPLAY_ONOFF | HD44780_DISPLAY_ON | HD44780_CURSOR_OFF | HD44780_CURSOR_NOBLINK); // wcz LCD, bez kursora i mrugania
}

//-------------------------------------------------------------------------------------------------
//
// Koniec pliku HD44780.c
//
//-------------------------------------------------------------------------------------------------
